@@ -1,12 +1,9 @@
 #pragma once
 
-#include <QMultiMap>
 #include <QString>
 #include <memory>
-#include "QObjectPtr.h"
 
 class MinecraftAccount;
-class QNetworkAccessManager;
 
 struct AuthSession {
     bool MakeOffline(QString offline_playername);
